/***************************************************************************
 *   Copyright (C) 2003 by Norman Jordan                                   *
 *   njordan@shaw.ca                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>

#include <gdk/gdk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include <panel-applet.h>
#include <gtk/gtk.h>

#include <libgnome/libgnome.h>

#include <glade/glade-xml.h>

#include "interface.h"
#include "gnome-netselect.h"
#include "browser_utils.h"
#include "configuration.h"

/* Global variables */
GdkPixbuf *normal_image = NULL;
GdkPixbuf *negative_image = NULL;
GtkWidget *event_box = NULL;
GtkWidget *image = NULL;
int last_button;
Window moz_win;
GladeXML *interface_desc = NULL;

/* Configuration Items */
gint browser_type;
gchar *browser_path = NULL;
gchar *normal_image_path = NULL;
gchar *negative_image_path = NULL;
gint display_action_type;
gint open_browser_action;
gint search_action;
gint goto_action;
gint search_engine;

/* Local global variables */
PanelApplet *applet_widget = NULL;

GtkWidget
*get_applet_widget()
{
  return GTK_WIDGET(applet_widget);
}

gint
get_applet_size()
{
  if(applet_widget == NULL)
  {
    return -1;
  }

  return panel_applet_get_size(applet_widget);
}

void
free_all_globals()
{
  GtkWidget *widget;

  if(interface_desc != NULL)
  {
    widget = glade_xml_get_widget(interface_desc, "prefs_window");
    gtk_widget_destroy(widget);
    g_object_unref(interface_desc);
    interface_desc = NULL;
  }
  if(normal_image != NULL)
  {
    g_object_unref(normal_image);
    normal_image = NULL;
  }
  if(negative_image != NULL)
  {
    g_object_unref(negative_image);
    negative_image = NULL;
  }
  if(applet_widget != NULL)
  {
    gtk_widget_destroy(GTK_WIDGET(applet_widget));
    applet_widget = NULL;
    image = NULL;
    event_box = NULL;
  }
  if(event_box != NULL)
  {
    gtk_widget_destroy(event_box);
    event_box = NULL;
    image = NULL;
  }
  if(image != NULL)
  {
    gtk_widget_destroy(image);
    image = NULL;
  }

  if(browser_path != NULL)
  {
    g_free(browser_path);
  }
  if(normal_image_path != NULL)
  {
    g_free(normal_image_path);
  }
  if(negative_image_path != NULL)
  {
    g_free(negative_image_path);
  }
  if(gconf_client != NULL)
  {
    g_object_unref(gconf_client);
    gconf_client = NULL;
  }
}

static gboolean
gnome_netselect_fill(PanelApplet *applet,
                     const gchar *iid,
                     gpointer     data)
{
  char *argv[] = {PACKAGE};

  if (strcmp (iid, "OAFIID:Gnome_NetSelect_Applet") != 0)
    return FALSE;

  gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, 1, argv, GNOME_PROGRAM_STANDARD_PROPERTIES, GNOME_PARAM_NONE);

  /* Initial the common data */
  InitAtoms();

  /* Initialize the configuration */
  last_button = -1;
  moz_win = None;

  if(initialize_configuration() == FALSE)
  {
    return FALSE;
  }

  if(create_applet(applet) == FALSE)
  {
    g_free(browser_path);
    return FALSE;
  }

  applet_widget = applet;

  return TRUE;
}


PANEL_APPLET_BONOBO_FACTORY("OAFIID:Gnome_NetSelect_Factory",
                            PANEL_TYPE_APPLET,
                            PACKAGE,
                            "0",
                            gnome_netselect_fill,
                            NULL);
