/***************************************************************************
 *   Copyright (C) 2003 by Norman Jordan                                   *
 *   njordan@shaw.ca                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <glade/glade-xml.h>

#include "interface.h"
#include "gnome-netselect.h"
#include "signal_handlers.h"
#include "configuration.h"

static BonoboUIVerb GnomeNetSelectMenuVerbs[] =
  {
    BONOBO_UI_UNSAFE_VERB("GnomeNetSelectProperties", show_preferences),
    BONOBO_UI_UNSAFE_VERB("GnomeNetSelectHelp", show_help),
    BONOBO_UI_UNSAFE_VERB("GnomeNetSelectAbout", show_about),
    BONOBO_UI_VERB_END
  };

gboolean create_applet(PanelApplet *applet)
{
  GdkPixbuf *pixbuf;
  guint size;

  /* Find the pixmaps */
  if(normal_image_path == NULL)
  {
    return FALSE;
  }

  if(negative_image_path == NULL)
  {
    g_free(normal_image_path);
    return FALSE;
  }

  /* Get panel details */
  size = panel_applet_get_size(applet);

  /* Create the widgets */
  event_box = gtk_event_box_new();

  /* Prepare the normal pixmap */
  pixbuf = gdk_pixbuf_new_from_file(normal_image_path, NULL);
  normal_image = gdk_pixbuf_scale_simple(pixbuf, size, size, GDK_INTERP_BILINEAR);
  g_object_unref(G_OBJECT(pixbuf));
  image = gtk_image_new_from_pixbuf(normal_image);

  /* Prepare the negative pixmap */
  pixbuf = gdk_pixbuf_new_from_file(negative_image_path, NULL);
  negative_image = gdk_pixbuf_scale_simple(pixbuf, size, size, GDK_INTERP_BILINEAR);
  g_object_unref(G_OBJECT(pixbuf));

  gtk_container_add(GTK_CONTAINER(event_box), image);

  gtk_widget_set_size_request(image, size, size);

  gtk_container_add (GTK_CONTAINER (applet), event_box);

  /* Add the signal handlers */
  gtk_signal_connect(GTK_OBJECT(event_box),
                     "button_press_event",
                     GTK_SIGNAL_FUNC(handle_button_event),
                     NULL);
  gtk_signal_connect(GTK_OBJECT(event_box),
                     "button_release_event",
                     GTK_SIGNAL_FUNC(handle_button_event),
                     NULL);
  gtk_signal_connect(GTK_OBJECT(applet),
                     "change-size",
                     GTK_SIGNAL_FUNC(handle_size_change_event),
                     NULL);
  gtk_signal_connect(GTK_OBJECT(applet),
                     "destroy",
                     GTK_SIGNAL_FUNC(handle_destroy_event),
                     NULL);

  panel_applet_setup_menu_from_file(applet,
                                    NULL,
                                    "gnome-netselect.xml",
                                    NULL,
                                    GnomeNetSelectMenuVerbs,
                                    NULL);

  gtk_widget_show_all(GTK_WIDGET(applet));

  return TRUE;
}

gboolean
prefs_window_deleted(GtkWidget *widget,
                     GdkEvent *event,
                     gpointer data)
{
  update_changed_settings();
  g_object_unref(interface_desc);
  interface_desc = NULL;

  return TRUE;
}

void
close_prefs_window_clicked(GtkButton *button,
                           gpointer data)
{
  GtkWidget *widget;

  update_changed_settings();

  widget = glade_xml_get_widget(interface_desc, "prefs_window");
  gtk_widget_destroy(widget);

  g_object_unref(interface_desc);
  interface_desc = NULL;
}

void
help_clicked(GtkButton *button,
             gpointer data)
{
  show_help(FALSE);
}

void
create_preferences_window()
{
  GtkWidget *window, *widget;

  if(interface_desc != NULL)
    return;

  interface_desc = glade_xml_new(DATADIR "/" PACKAGE "/glade/gnome-netselect_preferences.glade", NULL, NULL);

  /* Update the preferences window with the current configuration */
  widget = glade_xml_get_widget(interface_desc, "browser_type");
  gtk_option_menu_set_history(GTK_OPTION_MENU(widget), browser_type);
  glade_xml_signal_connect(interface_desc, "web_browser_changed", G_CALLBACK(web_browser_changed));

  widget = glade_xml_get_widget(interface_desc, "browser_path");
  gnome_file_entry_set_filename(GNOME_FILE_ENTRY(widget), browser_path);
  glade_xml_signal_connect(interface_desc, "browser_path_text_updated", G_CALLBACK(browser_path_text_updated));
  glade_xml_signal_connect(interface_desc, "browser_path_browse_done", G_CALLBACK(browser_path_browse_done));

  widget = glade_xml_get_widget(interface_desc, "search_engine");
  gtk_option_menu_set_history(GTK_OPTION_MENU(widget), search_engine);
  glade_xml_signal_connect(interface_desc, "search_engine_changed", G_CALLBACK(search_engine_changed));

  widget = glade_xml_get_widget(interface_desc, "open_in");
  gtk_option_menu_set_history(GTK_OPTION_MENU(widget), display_action_type);
  glade_xml_signal_connect(interface_desc, "open_in_changed", G_CALLBACK(open_in_changed));

  widget = glade_xml_get_widget(interface_desc, "open_browser");
  gtk_option_menu_set_history(GTK_OPTION_MENU(widget), open_browser_action);
  glade_xml_signal_connect(interface_desc, "open_browser_changed", G_CALLBACK(open_browser_changed));

  widget = glade_xml_get_widget(interface_desc, "open_url");
  gtk_option_menu_set_history(GTK_OPTION_MENU(widget), goto_action);
  glade_xml_signal_connect(interface_desc, "open_url_changed", G_CALLBACK(open_url_changed));

  widget = glade_xml_get_widget(interface_desc, "search");
  gtk_option_menu_set_history(GTK_OPTION_MENU(widget), search_action);
  glade_xml_signal_connect(interface_desc, "search_changed", G_CALLBACK(search_changed));

  widget = glade_xml_get_widget(interface_desc, "normal_icon_path");
  gnome_file_entry_set_filename(GNOME_FILE_ENTRY(widget), normal_image_path);
  glade_xml_signal_connect(interface_desc, "normal_icon_path_text_updated", G_CALLBACK(normal_icon_path_text_updated));
  glade_xml_signal_connect(interface_desc, "normal_icon_path_browse_done", G_CALLBACK(normal_icon_path_browse_done));

  widget = glade_xml_get_widget(interface_desc, "negative_icon_path");
  gnome_file_entry_set_filename(GNOME_FILE_ENTRY(widget), negative_image_path);
  glade_xml_signal_connect(interface_desc, "negative_icon_path_text_updated", G_CALLBACK(negative_icon_path_text_updated));
  glade_xml_signal_connect(interface_desc, "negative_icon_path_browse_done", G_CALLBACK(negative_icon_path_browse_done));

  glade_xml_signal_connect(interface_desc, "close_prefs_window_clicked", G_CALLBACK(close_prefs_window_clicked));

  glade_xml_signal_connect(interface_desc, "help_clicked", G_CALLBACK(help_clicked));

  window = glade_xml_get_widget(interface_desc, "prefs_window");
  glade_xml_signal_connect(interface_desc, "prefs_window_deleted", G_CALLBACK(prefs_window_deleted));
}
