/***************************************************************************
 *   Copyright (C) 2003 by Norman Jordan                                   *
 *   njordan@shaw.ca                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>
#include <panel-applet.h>
#include <string.h>

#include "signal_handlers.h"
#include "gnome-netselect.h"
#include "selection_utils.h"
#include "browser_utils.h"
#include "interface.h"

void
show_about()
{
  GtkWidget *about_box;
  static const gchar *authors[] = {"Norman Jordan <njordan@shaw.ca>", NULL};

  about_box = gnome_about_new("GnomeNetSelect",
                              "0.1",
                              "Copyright (c) Norman Jordan 2003",
                              _("Enhanced Mozilla launch button designed for Gnome"),
                              authors,
                              NULL,
                              NULL,
                              NULL);
  gtk_widget_show(about_box);
}

void
menu_show_help()
{
  show_help(TRUE);
}

void
show_help(gboolean from_menu)
{
  GError *error = NULL;

  if(!gnome_help_display("gnome-netselect", (from_menu) ? "toc" : "gnomenetselect-configuring", &error))
  {
    GtkWidget *msg_window = gtk_message_dialog_new(GTK_WINDOW(get_applet_widget()),
                                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                                   GTK_MESSAGE_ERROR,
                                                   GTK_BUTTONS_CLOSE,
                                                   "Couldn't find the Help documentation.");
    g_signal_connect_swapped(GTK_OBJECT(msg_window),
                             "close",
                             G_CALLBACK(gtk_widget_destroy),
                             GTK_OBJECT(msg_window));
    g_signal_connect_swapped(GTK_OBJECT(msg_window),
                             "response",
                             G_CALLBACK(gtk_widget_destroy),
                             GTK_OBJECT(msg_window));
    gtk_widget_show(msg_window);
    g_error_free(error);
  }
}

void
show_preferences()
{
  create_preferences_window();
}

guint
reset_image(gpointer data)
{
  gtk_image_set_from_pixbuf(GTK_IMAGE(image), normal_image);

  return FALSE;
}

void
selection_received(GtkClipboard *clipboard,
                   const gchar *text,
                   gpointer data)
{
  gchar *selection, *select, *prefix;

  selection = g_strdup(text);

  g_strstrip(selection);
  if(last_button == 1)
  {
    killstring(selection, " ");
    killstring(selection, NEWLINE_CHAR);
  }

  select = encode(selection, last_button);
  g_free(selection);
  if(select == NULL) {
    fprintf(stderr, "Could not allocate memory\n");
    return;
  }

  prefix = protocol(select, 0);

  switch(last_button) {
  case 1:
    URLSelection(prefix, select, display_action_type, 0);
    break;
  case 2:
    Search4Selection(select, display_action_type, 0);
    break;
  }
  last_button = -1;

  g_free(select);
  g_free(prefix);
}

void
open_browser()
{
  int status;
  gchar *command;
  Window win;
  int error;

  gtk_image_set_from_pixbuf(GTK_IMAGE(image), negative_image);
  g_timeout_add(2000, (GSourceFunc)reset_image, NULL);

  win = FindMozWin();
  if(win == None)
  {
    command = (gchar *)g_malloc(sizeof(gchar) * strlen(browser_path) + 3);
    strcpy(command, browser_path);
    g_strlcat(command, " &", strlen(browser_path) + 3);
    status = system(command);
    g_free(command);
  } else
  {
    if(display_action_type == 1)
    {
      error = SendCommand(win, "openURL(,new-window)");
    } else if(display_action_type == 2)
    {
      error = SendCommand(win, "openURL(,new-tab)");
    }
  }
}

void
open_action(gint action)
{
  GtkClipboard *clipboard;

  last_button = action;
  gtk_image_set_from_pixbuf(GTK_IMAGE(image), negative_image);
  g_timeout_add(2000, (GSourceFunc)reset_image, NULL);

  clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
  gtk_clipboard_request_text(clipboard, selection_received, NULL);
}

gint
handle_button_event(GtkWidget      *widget,
                    GdkEventButton *event,
                    gpointer       data)
{
  gint action_type;

  if(event->type == GDK_2BUTTON_PRESS && event->button == 1)
  {
    action_type = 0;
  } else if(event->type == GDK_BUTTON_RELEASE && event->button == 1 &&
    (event->state & GDK_SHIFT_MASK) != 0)
  {
    action_type = 2;
  } else if(event->type == GDK_BUTTON_RELEASE && event->button == 1 &&
    (event->state & GDK_CONTROL_MASK) != 0)
  {
    action_type = 1;
  } else
  {
    return 0;
  }

  if(open_browser_action == action_type)
    open_browser();
  else if(search_action == action_type)
    open_action(2);
  else if(goto_action == action_type)
    open_action(1);

  return 0;
}

void
handle_size_change_event(PanelApplet *applet,
                         gint arg1,
                         gpointer data)
{
  GdkPixbuf *pixbuf_normal, *pixbuf_scaled;
  int size;
  gchar *filename, *normal_path, *negative_path;

  filename = g_build_filename(PACKAGE, NORMAL_IMAGE, NULL);
  normal_path = gnome_pixmap_file(filename);
  g_free(filename);
  if(normal_path == NULL)
  {
    return;
  }

  filename = g_build_filename(PACKAGE, NEGATIVE_IMAGE, NULL);
  negative_path = gnome_pixmap_file(filename);
  g_free(filename);
  if(negative_path == NULL)
  {
    g_free(normal_path);
    return;
  }

  size = panel_applet_get_size(applet);

  pixbuf_normal = gdk_pixbuf_new_from_file(normal_path, NULL);
  g_free(normal_path);
  pixbuf_scaled = gdk_pixbuf_scale_simple(pixbuf_normal, size, size, GDK_INTERP_BILINEAR);
  g_object_unref(G_OBJECT(pixbuf_normal));
  gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf_scaled);
  g_object_unref(G_OBJECT(normal_image));
  normal_image = pixbuf_scaled;

  gtk_widget_set_size_request(image, size, size);

  pixbuf_normal = gdk_pixbuf_new_from_file(negative_path, NULL);
  g_free(negative_path);
  pixbuf_scaled = gdk_pixbuf_scale_simple(pixbuf_normal, size, size, GDK_INTERP_BILINEAR);
  g_object_unref(G_OBJECT(pixbuf_normal));
  g_object_unref(G_OBJECT(negative_image));
  negative_image = pixbuf_scaled;
}

gboolean
handle_destroy_event(GtkWidget *widget,
                     GdkEvent  *event,
                     gpointer data)
{
  free_all_globals();

  return TRUE;
}
